#include <iostream>
#include <sstream>
#include <cassert>

#include "po.h"         // Include the header file generated by LMX
#include "lmxparse.h"   // Include parser header file.  Only required where marshaling and unmarshaling is done.


// This is extracted from XML Schema Part 0.  The " characters have been modified to ' for easier
// -reading when embedded in a C++ string.
const char poXml[] =
    "<?xml version='1.0'?>\n"
    "<purchaseOrder orderDate='1999-10-20'>\n"
    "    <shipTo country='US'>\n"
    "        <name>Alice Smith</name>\n"
    "        <street>123 Maple Street</street>\n"
    "        <city>Mill Valley</city>\n"
    "        <state>CA</state>\n"
    "        <zip>90952</zip>\n"
    "    </shipTo>\n"
    "    <billTo country='US'>\n"
    "        <name>Robert Smith</name>\n"
    "        <street>8 Oak Avenue</street>\n"
    "        <city>Old Town</city>\n"
    "        <state>PA</state>\n"
    "        <zip>95819</zip>\n"
    "    </billTo>\n"
    "    <comment>Hurry, my lawn is going wild!</comment>\n"
    "    <items>\n"
    "        <item partNum='872-AA'>\n"
    "            <productName>Lawnmower</productName>\n"
    "            <quantity>1</quantity>\n"
    "            <USPrice>148.95</USPrice>\n"
    "            <comment>Confirm this is electric</comment>\n"
    "        </item>\n"
    "        <item partNum='926-AA'>\n"
    "            <productName>Baby Monitor</productName>\n"
    "            <quantity>1</quantity>\n"
    "            <USPrice>39.98</USPrice>\n"
    "            <shipDate>1999-05-21</shipDate>\n"
    "        </item>\n"
    "    </items>\n"
    "</purchaseOrder>\n";


int main( int argc, char **argv )
{
    try
    {
        // Read in the po XML
        //-------------------

        // Declare somehwere to store any error code
        lmx::elmx_error lmxError = lmx::ELMX_OK;

        // Construct an instance of Root from the XML contained in the memory at poXml
        Root po( poXml, sizeof( poXml ) - 1, &lmxError );

        if( lmxError != lmx::ELMX_OK )
        {
            std::cout << "An error occurred while reading XML\n";

            // To get further info in Debug mode do:
            LMX_OUTPUT_DEBUG_ERROR( std::cout );
            // Or:
            #if defined( LMX_DEBUG_CAPTURE_ERROR ) && LMX_DEBUG_CAPTURE_ERROR == 1
                std::cout << lmx::debug_error;
                // Or:
                std::string debugError( lmx::debug_error.to_string() );
            #endif

            return 1;
        }


        // Interrogate the parsed XML
        //---------------------------
        if( po.Getchosen() == Root::PurchaseOrderChosen )
        {
            // Make some intermediate variables so that components are easier to reference
            const PurchaseOrderType * pPoType = &po.GetPurchaseOrder();

            if( pPoType->IssetOrderDate() )
            {
                std::cout << "Ordered on: " << pPoType->GetOrderDate() << "\n";

                if( pPoType->GetOrderDate().get_year() <= 2000 )
                    std::cout << "This was ordered last century - It is now overdue!\n";
            }

            const Items * pItems = &pPoType->GetItems();

            float totalCost = 0;

            std::cout << "Number of items = " << pItems->SizeItem() << "\n";

            // Print out some information about each order item
            for( size_t i=0; i<pItems->SizeItem(); ++i )
            {
                std::cout << "Item " << (i + 1) << ": " <<
                            static_cast<int>(pItems->GetItem( i ).GetQuantity()) << " off - " <<
                            lmx::as_ascii( pItems->GetItem( i ).GetProductName() ) <<
                            " (" << lmx::as_ascii( pItems->GetItem( i ).GetPartNum() ) << ")";

                // The comment is optional, so only print it out if it is present
                if( pItems->GetItem( i ).IssetComment() )
                    std::cout << " [" << lmx::as_ascii( pItems->GetItem( i ).GetComment() ) << "]";

                std::cout << " - $" << pItems->GetItem( i ).GetUSPrice() << "\n";

                totalCost += pItems->GetItem( i ).GetUSPrice().get_scaled( 2 );
            }

            std::cout << "Total Cost: $" << totalCost/100 << "\n";

            std::cout << "Ship to: " << lmx::as_ascii( pPoType->GetShipTo().GetName() ) << "\n";
            std::cout << "Bill to: " << lmx::as_ascii( pPoType->GetBillTo().GetName() ) << "\n";
            std::cout << "\n";


            // Modify the XML
            //---------------
            Root altPo( po );    // We don't actually need to create a new instance before modifying

            altPo.GetPurchaseOrder().GetItems().AppendItem();
            Items::Item * pItem = &altPo.GetPurchaseOrder().GetItems().BackItem();

            pItem->SetProductName( L"Fence" );
            pItem->SetQuantity( 2 );
            pItem->SetPartNum( L"100-AB" );
            pItem->SetUSPrice( lmx::c_decimal( 12.95, 2U ) );

            // We can also do:
            // pItem->SetUSPrice( 12.95 );    // Although the schema needs to specify the fraction digits
            //                                // facet in order to get the number of decimal places correct!
            // pItem->SetUSPrice( lmx::c_decimal( 1295, 2 ) );    // e.g. 1295 / (10^2)
            // pItem->SetUSPrice( "12.95" );

            pItem->SetComment( L"Will this stop the baby getting on the lawn?" );

            assert( pItem->IsOccursOk() );    // Check sufficient elements and attributes added


            // Write the modified version of the XML
            //--------------------------------------
            std::cout << "\n\nModified XML output\n";

            if( altPo.Marshal( std::cout ) == lmx::ELMX_OK )
                std::cout << "Modified XML written successfully\n";
            else
                std::cout << "Error writing Modified XML\n";

            // Also marshal to a file
            if( altPo.Marshal( "po-out.xml" ) == lmx::ELMX_OK )
                std::cout << "Modified XML written successfully to file\n";
            else
                std::cout << "Error writing Modified XML to file\n";
        }


        // Write out the original XML
        //---------------------------
        std::cout << "\n\nXML output\n";

        if( po.Marshal( std::cout ) == lmx::ELMX_OK )
            std::cout << "Original XML written successfully\n";
        else
            std::cout << "Error writing Original XML\n";

    }   // Ends of try block

    catch( std::bad_alloc & )
    {
        // This is the most likely exception unless you have selected LMX to throw exceptions.
        std::cout << "An out of memory exception has occurred XML\n";
    }
    catch( ... )
    {
        // Other exceptions might include iostream exceptions etc.
        std::cout << "Another exception has occurred\n";
    }

    #ifdef _MSC_VER     // Allow for operation inside IDE
    std::cout << "----Press <Return> to continue----\n";
    std::cin.get();
    #endif

    return 0;
}
